@@ -20,11 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
 #include <string.h>
 
 #include "pango-attributes.h"
-#include "pango-impl-utils.h"
+#include "pango-utils.h"
 
 struct _PangoAttrList
 {
@@ -37,27 +36,25 @@ struct _PangoAttrIterator
 {
   GSList *next_attribute;
   GList *attribute_stack;
-  guint start_index;
-  guint end_index;
+  int start_index;
+  int end_index;
 };
 
-static PangoAttribute *pango_attr_color_new         (const PangoAttrClass *klass,
-                                                     guint16               red,
-                                                     guint16               green,
-                                                     guint16               blue);
-static PangoAttribute *pango_attr_string_new        (const PangoAttrClass *klass,
-                                                     const char           *str);
-static PangoAttribute *pango_attr_int_new           (const PangoAttrClass *klass,
-                                                     int                   value);
-static PangoAttribute *pango_attr_float_new         (const PangoAttrClass *klass,
-                                                     double                value);
-static PangoAttribute *pango_attr_size_new_internal (int                   size,
-						     gboolean              absolute);
+static PangoAttribute *pango_attr_color_new  (const PangoAttrClass *klass,
+                                              guint16               red,
+                                              guint16               green,
+                                              guint16               blue);
+static PangoAttribute *pango_attr_string_new (const PangoAttrClass *klass,
+                                              const char           *str);
+static PangoAttribute *pango_attr_int_new    (const PangoAttrClass *klass,
+                                              int                   value);
+static PangoAttribute *pango_attr_float_new  (const PangoAttrClass *klass,
+                                              double                value);
 
 
 /**
  * pango_attr_type_register:
- * @name: an identifier for the type (currently unused.)
+ * @name: an identifier for the type. (Currently unused.)
  * 
  * Allocate a new attribute type ID.
  * 
@@ -73,12 +70,11 @@ pango_attr_type_register (const gchar *name)
 
 /**
  * pango_attribute_copy:
- * @attr: a #PangoAttribute
+ * @attr: a #PangoAttribute.
  * 
  * Make a copy of an attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: a newly allocated #PangoAttribute.
  **/
 PangoAttribute *
 pango_attribute_copy (const PangoAttribute *attr)
@@ -141,10 +137,8 @@ pango_attr_string_copy (const PangoAttribute *attr)
 static void
 pango_attr_string_destroy (PangoAttribute *attr)
 {
-  PangoAttrString *sattr = (PangoAttrString *)attr;
-
-  g_free (sattr->value);
-  g_slice_free (PangoAttrString, sattr);
+  g_free (((PangoAttrString *)attr)->value);
+  g_free (attr);
 }
 
 static gboolean
@@ -158,7 +152,7 @@ static PangoAttribute *
 pango_attr_string_new (const PangoAttrClass *klass,
 		       const char           *str)
 {
-  PangoAttrString *result = g_slice_new (PangoAttrString);
+  PangoAttrString *result = g_new (PangoAttrString, 1);
 
   result->attr.klass = klass;
   result->value = g_strdup (str);
@@ -172,8 +166,7 @@ pango_attr_string_new (const PangoAttrClass *klass,
  * 
  * Create a new font family attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_family_new (const char *family)
@@ -193,15 +186,13 @@ pango_attr_family_new (const char *family)
 static PangoAttribute *
 pango_attr_language_copy (const PangoAttribute *attr)
 {
-  return pango_attr_language_new (((PangoAttrLanguage *)attr)->value);
+  return g_memdup (attr, sizeof (PangoAttrLanguage));
 }
 
 static void
 pango_attr_language_destroy (PangoAttribute *attr)
 {
-  PangoAttrLanguage *lattr = (PangoAttrLanguage *)attr;
-
-  g_slice_free (PangoAttrLanguage, lattr);
+  g_free (attr);
 }
 
 static gboolean
@@ -217,8 +208,7 @@ pango_attr_language_equal (const PangoAttribute *attr1,
  * 
  * Create a new language tag attribute. 
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_language_new (PangoLanguage *language)
@@ -234,7 +224,7 @@ pango_attr_language_new (PangoLanguage *language)
 
   g_return_val_if_fail (language != NULL, NULL);
   
-  result = g_slice_new (PangoAttrLanguage);
+  result = g_new (PangoAttrLanguage, 1);
 
   result->attr.klass = &klass;
   result->value = language;
@@ -256,9 +246,7 @@ pango_attr_color_copy (const PangoAttribute *attr)
 static void
 pango_attr_color_destroy (PangoAttribute *attr)
 {
-  PangoAttrColor *cattr = (PangoAttrColor *)attr;
-
-  g_slice_free (PangoAttrColor, cattr);
+  g_free (attr);
 }
 
 static gboolean
@@ -279,7 +267,7 @@ pango_attr_color_new (const PangoAttrClass *klass,
 		      guint16               green,
 		      guint16               blue)
 {
-  PangoAttrColor *result = g_slice_new (PangoAttrColor);
+  PangoAttrColor *result = g_new (PangoAttrColor, 1);
   result->attr.klass = klass;
   result->color.red = red;
   result->color.green = green;
@@ -296,8 +284,7 @@ pango_attr_color_new (const PangoAttrClass *klass,
  * 
  * Create a new foreground color attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_foreground_new (guint16 red,
@@ -322,8 +309,7 @@ pango_attr_foreground_new (guint16 red,
  * 
  * Create a new background color attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_background_new (guint16 red,
@@ -351,9 +337,7 @@ pango_attr_int_copy (const PangoAttribute *attr)
 static void
 pango_attr_int_destroy (PangoAttribute *attr)
 {
-  PangoAttrInt *iattr = (PangoAttrInt *)attr;
-
-  g_slice_free (PangoAttrInt, iattr);
+  g_free (attr);
 }
 
 static gboolean
@@ -370,7 +354,7 @@ static PangoAttribute *
 pango_attr_int_new (const PangoAttrClass *klass,
 		    int                   value)
 {
-  PangoAttrInt *result = g_slice_new (PangoAttrInt);
+  PangoAttrInt *result = g_new (PangoAttrInt, 1);
   result->attr.klass = klass;
   result->value = value;
 
@@ -388,9 +372,7 @@ pango_attr_float_copy (const PangoAttribute *attr)
 static void
 pango_attr_float_destroy (PangoAttribute *attr)
 {
-  PangoAttrFloat *fattr = (PangoAttrFloat *)attr;
-
-  g_slice_free (PangoAttrFloat, fattr);
+  g_free (attr);
 }
 
 static gboolean
@@ -407,99 +389,32 @@ static PangoAttribute*
 pango_attr_float_new  (const PangoAttrClass *klass,
                        double                value)
 {
-  PangoAttrFloat *result = g_slice_new (PangoAttrFloat);
+  PangoAttrFloat *result = g_new (PangoAttrFloat, 1);
   result->attr.klass = klass;
   result->value = value;
 
   return (PangoAttribute *)result;
 }
 
-static PangoAttribute *
-pango_attr_size_copy (const PangoAttribute *attr)
-{
-  const PangoAttrSize *size_attr = (PangoAttrSize *)attr;
-
-  if (attr->klass->type == PANGO_ATTR_ABSOLUTE_SIZE)
-    return pango_attr_size_new_absolute (size_attr->size);
-  else
-    return pango_attr_size_new (size_attr->size);
-}
-
-static void
-pango_attr_size_destroy (PangoAttribute *attr)
-{
-  PangoAttrSize *sattr = (PangoAttrSize *)attr;
-
-  g_slice_free (PangoAttrSize, sattr);
-}
-
-static gboolean
-pango_attr_size_equal (const PangoAttribute *attr1,
-		       const PangoAttribute *attr2)
-{
-  const PangoAttrSize *size_attr1 = (const PangoAttrSize *)attr1;
-  const PangoAttrSize *size_attr2 = (const PangoAttrSize *)attr2;
-  
-  return size_attr1->size == size_attr2->size;
-}
-
-static PangoAttribute *
-pango_attr_size_new_internal (int size,
-			      gboolean absolute)
-{
-  PangoAttrSize *result;
-  
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_SIZE,
-    pango_attr_size_copy,
-    pango_attr_size_destroy,
-    pango_attr_size_equal
-  };
-  static const PangoAttrClass absolute_klass = {
-    PANGO_ATTR_ABSOLUTE_SIZE,
-    pango_attr_size_copy,
-    pango_attr_size_destroy,
-    pango_attr_size_equal
-  };
-
-  result = g_slice_new (PangoAttrSize);
-  result->attr.klass = absolute ? &absolute_klass : &klass;
-  result->size = size;
-  result->absolute = absolute;
-
-  return (PangoAttribute *)result;
-}
-
 /**
  * pango_attr_size_new:
- * @size: the font size, in %PANGO_SCALE<!-- -->ths of a point.
+ * @size: the font size, in #PANGO_SCALE<!-- -->ths of a point.
  * 
- * Create a new font-size attribute in fractional points.
+ * Create a new font-size attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_size_new (int size)
 {
-  return pango_attr_size_new_internal (size, FALSE);
-}
+  static const PangoAttrClass klass = {
+    PANGO_ATTR_SIZE,
+    pango_attr_int_copy,
+    pango_attr_int_destroy,
+    pango_attr_int_equal
+  };
 
-/**
- * pango_attr_size_new_absolute:
- * @size: the font size, in %PANGO_SCALE<!-- -->ths of a device unit.
- * 
- * Create a new font-size attribute in device units.
- * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
- *
- * Since: 1.8
- **/
-PangoAttribute *
-pango_attr_size_new_absolute (int size)
-{
-  return pango_attr_size_new_internal (size, TRUE);
+  return pango_attr_int_new (&klass, size);
 }
 
 /**
@@ -508,8 +423,7 @@ pango_attr_size_new_absolute (int size)
  * 
  * Create a new font slant style attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_style_new (PangoStyle style)
@@ -530,8 +444,7 @@ pango_attr_style_new (PangoStyle style)
  * 
  * Create a new font weight attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_weight_new (PangoWeight weight)
@@ -552,8 +465,7 @@ pango_attr_weight_new (PangoWeight weight)
  *
  * Create a new font variant attribute (normal or small caps)
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_variant_new (PangoVariant variant)
@@ -574,8 +486,7 @@ pango_attr_variant_new (PangoVariant variant)
  * 
  * Create a new font stretch attribute
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_stretch_new (PangoStretch  stretch)
@@ -604,7 +515,7 @@ pango_attr_font_desc_destroy (PangoAttribute *attr)
   PangoAttrFontDesc *desc_attr = (PangoAttrFontDesc *)attr;
 
   pango_font_description_free (desc_attr->desc);
-  g_slice_free (PangoAttrFontDesc, desc_attr);
+  g_free (attr);
 }
 
 static gboolean
@@ -619,14 +530,13 @@ pango_attr_font_desc_equal (const PangoAttribute *attr1,
 
 /**
  * pango_attr_font_desc_new:
- * @desc: the font description
+ * @desc: 
  * 
- * Create a new font description attribute. This attribute
+ * Create a new font description attribute. (This attribute
  * allows setting family, style, weight, variant, stretch,
- * and size simultaneously.
+ * and size simultaneously.)
  * 
- * Return value:  the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: 
  **/
 PangoAttribute *
 pango_attr_font_desc_new (const PangoFontDescription *desc)
@@ -638,7 +548,7 @@ pango_attr_font_desc_new (const PangoFontDescription *desc)
     pango_attr_font_desc_equal
   };
 
-  PangoAttrFontDesc *result = g_slice_new (PangoAttrFontDesc);
+  PangoAttrFontDesc *result = g_new (PangoAttrFontDesc, 1);
   result->attr.klass = &klass;
   result->desc = pango_font_description_copy (desc);
 
@@ -650,10 +560,9 @@ pango_attr_font_desc_new (const PangoFontDescription *desc)
  * pango_attr_underline_new:
  * @underline: the underline style.
  * 
- * Create a new underline-style attribute.
+ * Create a new underline-style object.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_underline_new (PangoUnderline underline)
@@ -668,44 +577,13 @@ pango_attr_underline_new (PangoUnderline underline)
   return pango_attr_int_new (&klass, (int)underline);
 }
 
-/**
- * pango_attr_underline_color_new:
- * @red: the red value (ranging from 0 to 65535)
- * @green: the green value
- * @blue: the blue value
- * 
- * Create a new underline color attribute. This attribute
- * modifies the color of underlines. If not set, underlines
- * will use the foreground color.
- * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
- *
- * Since: 1.8
- **/
-PangoAttribute *
-pango_attr_underline_color_new (guint16 red,
-				guint16 green,
-				guint16 blue)
-{
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_UNDERLINE_COLOR,
-    pango_attr_color_copy,
-    pango_attr_color_destroy,
-    pango_attr_color_equal
-  };
-
-  return pango_attr_color_new (&klass, red, green, blue);
-}
-
 /**
  * pango_attr_strikethrough_new:
  * @strikethrough: %TRUE if the text should be struck-through.
  * 
- * Create a new strike-through attribute.
+ * Create a new font strike-through attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_strikethrough_new (gboolean strikethrough)
@@ -720,45 +598,15 @@ pango_attr_strikethrough_new (gboolean strikethrough)
   return pango_attr_int_new (&klass, (int)strikethrough);
 }
 
-/**
- * pango_attr_strikethrough_color_new:
- * @red: the red value (ranging from 0 to 65535)
- * @green: the green value
- * @blue: the blue value
- * 
- * Create a new strikethrough color attribute. This attribute
- * modifies the color of strikethrough lines. If not set, strikethrough
- * lines will use the foreground color.
- * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
- *
- * Since: 1.8
- **/
-PangoAttribute *
-pango_attr_strikethrough_color_new (guint16 red,
-				    guint16 green,
-				    guint16 blue)
-{
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_STRIKETHROUGH_COLOR,
-    pango_attr_color_copy,
-    pango_attr_color_destroy,
-    pango_attr_color_equal
-  };
-
-  return pango_attr_color_new (&klass, red, green, blue);
-}
-
 /**
  * pango_attr_rise_new:
  * @rise: the amount that the text should be displaced vertically,
- *        in Pango units. Positive values displace the text upwards.
+ *        in 10'000ths of an em. Positive values displace the
+ *        text upwards.
  * 
  * Create a new baseline displacement attribute.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute *
 pango_attr_rise_new (int rise)
@@ -780,8 +628,7 @@ pango_attr_rise_new (int rise)
  * Create a new font size scale attribute. The base font for the
  * affected text will have its size multiplied by @scale_factor.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
+ * Return value: the new #PangoAttribute.
  **/
 PangoAttribute*
 pango_attr_scale_new (double scale_factor)
@@ -796,85 +643,18 @@ pango_attr_scale_new (double scale_factor)
   return pango_attr_float_new (&klass, scale_factor);
 }
 
-/**
- * pango_attr_fallback_new:
- * @enable_fallback: %TRUE if we should fall back on other fonts 
- *                   for characters the active font is missing.
- * 
- * Create a new font fallback attribute.
- *
- * If fallback is disabled, characters will only be used from the
- * closest matching font on the system. No fallback will be done to
- * other fonts on the system that might contain the characters in the
- * text.
- * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
- *
- * Since: 1.4
- **/
-PangoAttribute *
-pango_attr_fallback_new (gboolean enable_fallback)
-{
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_FALLBACK,
-    pango_attr_int_copy,
-    pango_attr_int_destroy,
-    pango_attr_int_equal,
-  };
-
-  return pango_attr_int_new (&klass, (int)enable_fallback);
-}
-
-/**
- * pango_attr_letter_spacing_new:
- * @letter_spacing: amount of extra space to add between graphemes
- *   of the text, in Pango units.
- * 
- * Create a new letter-spacing attribute.
- * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
- *
- * Since: 1.6
- **/
-PangoAttribute *
-pango_attr_letter_spacing_new (int letter_spacing)
-{
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_LETTER_SPACING,
-    pango_attr_int_copy,
-    pango_attr_int_destroy,
-    pango_attr_int_equal
-  };
-
-  return pango_attr_int_new (&klass, letter_spacing);
-}
-
 static PangoAttribute *
 pango_attr_shape_copy (const PangoAttribute *attr)
 {
   const PangoAttrShape *shape_attr = (PangoAttrShape *)attr;
-  gpointer data;
-
-  if (shape_attr->copy_func)
-    data = shape_attr->copy_func (shape_attr->data);
-  else
-    data = shape_attr->data;
   
-  return pango_attr_shape_new_with_data (&shape_attr->ink_rect, &shape_attr->logical_rect,
-					 data, shape_attr->copy_func, shape_attr->destroy_func);
+  return pango_attr_shape_new (&shape_attr->ink_rect, &shape_attr->logical_rect);
 }
 
 static void
 pango_attr_shape_destroy (PangoAttribute *attr)
 {
-  PangoAttrShape *shape_attr = (PangoAttrShape *)attr;
-  
-  if (shape_attr->destroy_func)
-    shape_attr->destroy_func (shape_attr->data);
-  
-  g_slice_free (PangoAttrShape, shape_attr);
+  g_free (attr);
 }
 
 static gboolean
@@ -883,7 +663,7 @@ pango_attr_shape_equal (const PangoAttribute *attr1,
 {
   const PangoAttrShape *shape_attr1 = (const PangoAttrShape *)attr1;
   const PangoAttrShape *shape_attr2 = (const PangoAttrShape *)attr2;
-
+  
   return (shape_attr1->logical_rect.x == shape_attr2->logical_rect.x &&
 	  shape_attr1->logical_rect.y == shape_attr2->logical_rect.y &&
 	  shape_attr1->logical_rect.width == shape_attr2->logical_rect.width &&
@@ -891,36 +671,24 @@ pango_attr_shape_equal (const PangoAttribute *attr1,
 	  shape_attr1->ink_rect.x == shape_attr2->ink_rect.x &&
 	  shape_attr1->ink_rect.y == shape_attr2->ink_rect.y &&
 	  shape_attr1->ink_rect.width == shape_attr2->ink_rect.width &&
-	  shape_attr1->ink_rect.height == shape_attr2->ink_rect.height &&
-	  shape_attr1->data == shape_attr2->data);
+	  shape_attr1->ink_rect.height == shape_attr2->ink_rect.height);
 }
 
 /**
- * pango_attr_shape_new_with_data:
+ * pango_attr_shape_new:
  * @ink_rect:     ink rectangle to assign to each character
- * @logical_rect: logical rectangle to assign to each character
- * @data:         user data pointer
- * @copy_func:    function to copy @data when the attribute
- *                is copied. If %NULL, @data is simply copied
- *                as a pointer.
- * @destroy_func: function to free @data when the attribute
- *                is freed, or %NULL
+ * @logical_rect: logical rectangle assign to each character
  * 
- * Like pango_attr_shape_new(), but a user data pointer is also
- * provided; this pointer can be accessed when later
- * rendering the glyph.
+ * Create a new shape attribute. A shape is used to impose a
+ * particular ink and logical rect on the result of shaping a
+ * particular glyph. This might be used, for instance, for
+ * embedding a picture or a widget inside a PangoLayout.
  * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
- *
- * Since: 1.8
+ * Return value: the newly created attribute
  **/
 PangoAttribute *
-pango_attr_shape_new_with_data (const PangoRectangle  *ink_rect,
-				const PangoRectangle  *logical_rect,
-				gpointer               data,
-				PangoAttrDataCopyFunc  copy_func,
-				GDestroyNotify         destroy_func)
+pango_attr_shape_new (const PangoRectangle *ink_rect,
+		      const PangoRectangle *logical_rect)
 {
   static const PangoAttrClass klass = {
     PANGO_ATTR_SHAPE,
@@ -934,48 +702,21 @@ pango_attr_shape_new_with_data (const PangoRectangle  *ink_rect,
   g_return_val_if_fail (ink_rect != NULL, NULL);
   g_return_val_if_fail (logical_rect != NULL, NULL);
   
-  result = g_slice_new (PangoAttrShape);
+  result = g_new (PangoAttrShape, 1);
   result->attr.klass = &klass;
   result->ink_rect = *ink_rect;
   result->logical_rect = *logical_rect;
-  result->data = data;
-  result->copy_func = copy_func;
-  result->destroy_func =  destroy_func;
 
   return (PangoAttribute *)result;
 }
 
-/**
- * pango_attr_shape_new:
- * @ink_rect:     ink rectangle to assign to each character
- * @logical_rect: logical rectangle to assign to each character
- * 
- * Create a new shape attribute. A shape is used to impose a
- * particular ink and logical rect on the result of shaping a
- * particular glyph. This might be used, for instance, for
- * embedding a picture or a widget inside a PangoLayout.
- * 
- * Return value: the newly allocated #PangoAttribute, which should be
- *               freed with pango_attribute_destroy().
- **/
-PangoAttribute *
-pango_attr_shape_new (const PangoRectangle *ink_rect,
-		      const PangoRectangle *logical_rect)
-{
-  g_return_val_if_fail (ink_rect != NULL, NULL);
-  g_return_val_if_fail (logical_rect != NULL, NULL);
-
-  return pango_attr_shape_new_with_data (ink_rect, logical_rect,
-					 NULL, NULL, NULL);
-}
-
 GType
 pango_attr_list_get_type (void)
 {
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static (I_("PangoAttrList"),
+    our_type = g_boxed_type_register_static ("PangoAttrList",
 					     (GBoxedCopyFunc) pango_attr_list_copy,
 					     (GBoxedFreeFunc) pango_attr_list_unref);
 
@@ -985,15 +726,14 @@ pango_attr_list_get_type (void)
 /**
  * pango_attr_list_new:
  * 
- * Create a new empty attribute list with a reference count of one.
+ * Create a new empty attribute list with a reference count of 1.
  * 
- * Return value: the newly allocated #PangoAttrList, which should
- *               be freed with pango_attr_list_unref().
+ * Return value: the newly allocated #PangoAttrList.
  **/
 PangoAttrList *
 pango_attr_list_new (void)
 {
-  PangoAttrList *list = g_slice_new (PangoAttrList);
+  PangoAttrList *list = g_new (PangoAttrList, 1);
 
   list->ref_count = 1;
   list->attributes = NULL;
@@ -1007,19 +747,13 @@ pango_attr_list_new (void)
  * @list: a #PangoAttrList
  * 
  * Increase the reference count of the given attribute list by one.
- *
- * Return value: The attribute list passed in
- *
- * Since: 1.10
  **/
-PangoAttrList *
+void
 pango_attr_list_ref (PangoAttrList *list)
 {
-  g_return_val_if_fail (list != NULL, NULL);
+  g_return_if_fail (list != NULL);
 
   list->ref_count++;
-
-  return list;
 }
 
 /**
@@ -1052,7 +786,7 @@ pango_attr_list_unref (PangoAttrList *list)
 
       g_slist_free (list->attributes);
 
-      g_slice_free (PangoAttrList, list);
+      g_free (list);
     }
 }
 
@@ -1060,11 +794,9 @@ pango_attr_list_unref (PangoAttrList *list)
  * pango_attr_list_copy:
  * @list: a #PangoAttrList
  * 
- * Copy @list and return an identical new list.
+ * Copy @list and return an identical, new list.
  *
- * Return value: the newly allocated #PangoAttrList, with a
- *               reference count of one, which should
- *               be freed with pango_attr_list_unref().
+ * Return value: new attribute list
  **/
 PangoAttrList *
 pango_attr_list_copy (PangoAttrList *list)
@@ -1100,7 +832,7 @@ pango_attr_list_insert_internal (PangoAttrList  *list,
 				 gboolean        before)
 {
   GSList *tmp_list, *prev, *link;
-  guint start_index = attr->start_index;
+  gint start_index = attr->start_index;
 
   if (!list->attributes)
     {
@@ -1207,16 +939,10 @@ pango_attr_list_change (PangoAttrList  *list,
 			PangoAttribute *attr)
 {
   GSList *tmp_list, *prev, *link;
-  guint start_index = attr->start_index;
-  guint end_index = attr->end_index;
-
+  gint start_index = attr->start_index;
+  gint end_index = attr->end_index;
+  
   g_return_if_fail (list != NULL);
-
-  if (start_index == end_index)	/* empty, nothing to do */
-    {
-      pango_attribute_destroy (attr);
-      return;
-    }
   
   tmp_list = list->attributes;
   prev = NULL;
@@ -1401,13 +1127,11 @@ pango_attr_list_change (PangoAttrList  *list,
  *       must be specified since the attributes in @other
  *       may only be present at some subsection of this range)
  * 
- * This function opens up a hole in @list, fills it in with attributes from
- * the left, and then merges @other on top of the hole.
- *
+ * This function splices attribute list @other into @list.
  * This operation is equivalent to stretching every attribute
- * that applies at position @pos in @list by an amount @len,
+ * applies at position @pos in @list by an amount @len,
  * and then calling pango_attr_list_change() with a copy
- * of each attribute in @other in sequence (offset in position by @pos).
+ * of each attributes in @other in sequence (offset in position by @pos).
  *
  * This operation proves useful for, for instance, inserting
  * a preedit string in the middle of an edit buffer.
@@ -1419,40 +1143,26 @@ pango_attr_list_splice (PangoAttrList *list,
 			gint           len)
 {
   GSList *tmp_list;
-  guint upos, ulen;
   
   g_return_if_fail (list != NULL);
   g_return_if_fail (other != NULL); 
   g_return_if_fail (pos >= 0);
   g_return_if_fail (len >= 0);
-
-  upos = (guint)pos;
-  ulen = (guint)len;
-
-/* This definition only works when a and b are unsigned; overflow
- * isn't defined in the C standard for signed integers
- */
-#define CLAMP_ADD(a,b) (((a) + (b) < (a)) ? G_MAXUINT : (a) + (b))
-  
+ 
   tmp_list = list->attributes;
   while (tmp_list)
     {
       PangoAttribute *attr = tmp_list->data;
 
-      if (attr->start_index <= upos)
+      if (attr->start_index <= pos)
 	{
-	  if (attr->end_index > upos)
-	    attr->end_index = CLAMP_ADD (attr->end_index, ulen);
+	  if (attr->end_index > pos)
+	    attr->end_index += len;
 	}
       else
 	{
-	  /* This could result in a zero length attribute if it
-	   * gets squashed up against G_MAXUINT, but deleting such
-	   * an element could (in theory) suprise the caller, so
-	   * we don't delete it.
-	   */
-	  attr->start_index = CLAMP_ADD (attr->start_index, ulen);
-	  attr->end_index = CLAMP_ADD (attr->end_index, ulen);
+	  attr->start_index += len;
+	  attr->end_index += len;
 	}
 
       tmp_list = tmp_list->next;
@@ -1462,17 +1172,13 @@ pango_attr_list_splice (PangoAttrList *list,
   while (tmp_list)
     {
       PangoAttribute *attr = pango_attribute_copy (tmp_list->data);
-      attr->start_index = CLAMP_ADD (attr->start_index, upos);
-      attr->end_index = CLAMP_ADD (attr->end_index, upos);
+      attr->start_index += pos;
+      attr->end_index += pos;
 
-      /* Same as above, the attribute could be squashed to zero-length; here
-       * pango_attr_list_change() will take care of deleting it.
-       */
       pango_attr_list_change (list, attr);
       
       tmp_list = tmp_list->next;
     }
-#undef CLAMP_ADD
 }
 
 /**
@@ -1480,10 +1186,9 @@ pango_attr_list_splice (PangoAttrList *list,
  * @list: a #PangoAttrList
  * 
  * Create a iterator initialized to the beginning of the list.
- * @list must not be modified until this iterator is freed.
  * 
- * Return value: the newly allocated #PangoAttrIterator, which should
- *               be freed with pango_attr_iterator_destroy().
+ * Return value: a new #PangoIterator. @list must not be modified
+ *               until this iterator is freed with pango_attr_iterator_destroy().
  **/
 PangoAttrIterator *
 pango_attr_list_get_iterator (PangoAttrList  *list)
@@ -1492,7 +1197,7 @@ pango_attr_list_get_iterator (PangoAttrList  *list)
 
   g_return_val_if_fail (list != NULL, NULL);
 
-  iterator = g_slice_new (PangoAttrIterator);
+  iterator = g_new (PangoAttrIterator, 1);
   iterator->next_attribute = list->attributes;
   iterator->attribute_stack = NULL;
 
@@ -1500,7 +1205,7 @@ pango_attr_list_get_iterator (PangoAttrList  *list)
   iterator->end_index = 0;
 
   if (!pango_attr_iterator_next (iterator))
-    iterator->end_index = G_MAXUINT;
+    iterator->end_index = G_MAXINT;
 
   return iterator;
 }
@@ -1511,11 +1216,7 @@ pango_attr_list_get_iterator (PangoAttrList  *list)
  * @start: location to store the start of the range
  * @end: location to store the end of the range
  * 
- * Get the range of the current segment. Note that the
- * stored return values are signed, not unsigned like
- * the values in #PangoAttribute. To deal with this API
- * oversight, stored return values that wouldn't fit into
- * a signed integer are clamped to %G_MAXINT.
+ * Get the range of the current segment.
  **/
 void
 pango_attr_iterator_range (PangoAttrIterator *iterator,
@@ -1525,9 +1226,9 @@ pango_attr_iterator_range (PangoAttrIterator *iterator,
   g_return_if_fail (iterator != NULL);
 
   if (start)
-    *start = MIN (iterator->start_index, G_MAXINT);
+    *start = iterator->start_index;
   if (end)
-    *end = MIN (iterator->end_index, G_MAXINT);
+    *end = iterator->end_index;
 }
 
 /**
@@ -1549,7 +1250,7 @@ pango_attr_iterator_next (PangoAttrIterator *iterator)
     return FALSE;
 
   iterator->start_index = iterator->end_index;
-  iterator->end_index = G_MAXUINT;
+  iterator->end_index = G_MAXINT;
   
   tmp_list = iterator->attribute_stack;
   while (tmp_list)
@@ -1573,11 +1274,8 @@ pango_attr_iterator_next (PangoAttrIterator *iterator)
   while (iterator->next_attribute &&
 	 ((PangoAttribute *)iterator->next_attribute->data)->start_index == iterator->start_index)
     {
-      if (((PangoAttribute *)iterator->next_attribute->data)->end_index > iterator->start_index)
-	{
-	  iterator->attribute_stack = g_list_prepend (iterator->attribute_stack, iterator->next_attribute->data);
-	  iterator->end_index = MIN (iterator->end_index, ((PangoAttribute *)iterator->next_attribute->data)->end_index);
-	}
+      iterator->attribute_stack = g_list_prepend (iterator->attribute_stack, iterator->next_attribute->data);
+      iterator->end_index = MIN (iterator->end_index, ((PangoAttribute *)iterator->next_attribute->data)->end_index);
       iterator->next_attribute = iterator->next_attribute->next;
     }
 
@@ -1593,8 +1291,7 @@ pango_attr_iterator_next (PangoAttrIterator *iterator)
  * 
  * Copy a #PangoAttrIterator
  *
- * Return value: the newly allocated #PangoAttrIterator, which should
- *               be freed with pango_attr_iterator_destroy().
+ * Return value: Copy of @iterator
  **/
 PangoAttrIterator *
 pango_attr_iterator_copy (PangoAttrIterator *iterator)
@@ -1603,7 +1300,7 @@ pango_attr_iterator_copy (PangoAttrIterator *iterator)
 
   g_return_val_if_fail (iterator != NULL, NULL);
 
-  copy = g_slice_new (PangoAttrIterator);
+  copy = g_new (PangoAttrIterator, 1);
 
   *copy = *iterator;
 
@@ -1624,7 +1321,7 @@ pango_attr_iterator_destroy (PangoAttrIterator *iterator)
   g_return_if_fail (iterator != NULL);
   
   g_list_free (iterator->attribute_stack);
-  g_slice_free (PangoAttrIterator, iterator);
+  g_free (iterator);
 }
 
 /**
@@ -1668,12 +1365,12 @@ pango_attr_iterator_get (PangoAttrIterator *iterator,
  * @iterator: a #PangoAttrIterator
  * @desc: a #PangoFontDescription to fill in with the current values.
  *        The family name in this structure will be set using
- *        pango_font_description_set_family_static() using values from
+ *        pango_font_description_set_family_static using values from
  *        an attribute in the #PangoAttrList associated with the iterator,
  *        so if you plan to keep it around, you must call:
  *        <literal>pango_font_description_set_family (desc, pango_font_description_get_family (desc))</literal>.
  * @language: if non-%NULL, location to store language tag for item, or %NULL
- *            if none is found.
+ *            if non is found.
  * @extra_attrs: if non-%NULL, location in which to store a list of non-font
  *           attributes at the the current position; only the highest priority
  *           value of each attribute will be added to this list. In order
@@ -1761,14 +1458,7 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
 	  if (!(mask & PANGO_FONT_MASK_SIZE))
 	    {
 	      mask |= PANGO_FONT_MASK_SIZE;
-	      pango_font_description_set_size (desc, ((PangoAttrSize *)attr)->size);
-	    }
-	  break;
-	case PANGO_ATTR_ABSOLUTE_SIZE:
-	  if (!(mask & PANGO_FONT_MASK_SIZE))
-	    {
-	      mask |= PANGO_FONT_MASK_SIZE;
-	      pango_font_description_set_absolute_size (desc, ((PangoAttrSize *)attr)->size);
+	      pango_font_description_set_size (desc, ((PangoAttrInt *)attr)->value);
 	    }
 	  break;
         case PANGO_ATTR_SCALE:
@@ -1828,10 +1518,8 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
  * of @list for which @func returns %TRUE and inserts them into
  * a new list.
  * 
- * Return value: the new #PangoAttrList or %NULL if
+ * Return value: a newly allocated #PangoAttrList or %NULL if
  *  no attributes of the given types were found.
- *
- * Since: 1.2
  **/
 PangoAttrList *
 pango_attr_list_filter (PangoAttrList       *list,
@@ -1842,11 +1530,13 @@ pango_attr_list_filter (PangoAttrList       *list,
   PangoAttrList *new = NULL;
   GSList *tmp_list;
   GSList *prev;
+  GSList *new_attrs;
   
   g_return_val_if_fail (list != NULL, NULL);
 
   tmp_list = list->attributes;
   prev = NULL;
+  new_attrs = NULL;
   while (tmp_list)
     {
       GSList *next = tmp_list->next;
@@ -1889,16 +1579,14 @@ pango_attr_list_filter (PangoAttrList       *list,
 
 /**
  * pango_attr_iterator_get_attrs:
- * @iterator: a #PangoAttrIterator
+ * @iterator: a #PangAttrIterator
  * 
- * Gets a list of all attributes at the current position of the
+ * Gets a list all attributes a the current position of the
  * iterator.
  * 
  * Return value: a list of all attributes for the current range.
  *   To free this value, call pango_attribute_destroy() on
  *   each value and g_slist_free() on the list.
- *
- * Since: 1.2
  **/
 GSList *
 pango_attr_iterator_get_attrs (PangoAttrIterator *iterator)
